@@ -7,7 +7,7 @@ struct World
     commandment: String
 }
 
-#[tokio::main]
+#[tokio::main(flavor = "current_thread")]
 async fn main() -> Result<(), AteError>
 {
     // The default configuration will store the redo log locally in the temporary folder
@@ -20,8 +20,8 @@ async fn main() -> Result<(), AteError>
     let chain = builder.open(&ChainKey::from("universe")).await?;
     
     // We interact with the data stored in the chain-of-trust using a DIO
-    let session = AteSession::new(&conf);
-    let mut dio = chain.dio(&session).await;
+    let session = AteSessionUser::new();
+    let dio = chain.dio_mut(&session).await;
     
     // In this example we store some data in the "World" object
     let key = dio.store(World {
