@@ -1,5 +1,5 @@
 #![allow(unused_imports)]
-use log::{error, info, debug};
+use tracing::{error, info, debug};
 use std::sync::Arc;
 
 use fxhash::FxHashSet;
@@ -58,7 +58,7 @@ for AntiReplayPlugin
     fn validate(&self, header: &EventHeader, _conversation: Option<&Arc<ConversationSession>>) -> Result<ValidationResult, ValidationError> {
         match self.seen.contains(&header.raw.event_hash) {
             true => {
-                #[cfg(feature = "verbose")]
+                #[cfg(feature = "enable_verbose")]
                 debug!("rejected event as it is a duplicate - {}", header.raw.event_hash);
                 Ok(ValidationResult::Deny)
             },
