@@ -1,10 +1,11 @@
 #[allow(unused_imports)]
-use log::{info, error, debug};
+use tracing::{info, warn, debug, error, trace, instrument, span, Level};
 
 use crate::event::*;
 
 use tokio::sync::mpsc;
 
+#[derive(Debug)]
 pub(crate) struct ChainListener
 {
     pub(crate) id: u64,
