@@ -6,8 +6,9 @@ mod protected_async;
 mod protected_sync;
 mod workers;
 mod compact;
-#[cfg(feature = "rotate")]
+#[cfg(feature = "enable_rotate")]
 mod rotate;
+mod backup;
 
 pub use self::core::*;
 pub use new::*;
@@ -15,5 +16,6 @@ pub use compact::*;
 pub(crate) use listener::*;
 pub(crate) use protected_async::*;
 pub(crate) use protected_sync::*;
+pub(crate) use workers::*;
 
 pub use crate::trust::ChainKey;
\ No newline at end of file
