@@ -6,6 +6,7 @@ pub mod remove_duplicates;
 pub mod tombstone_compactor;
 pub mod cut_off_compactor;
 pub mod sig_compactor;
+pub mod public_key_compactor;
 mod tests;
 
 pub(crate) use compact_state::*;
@@ -16,4 +17,5 @@ pub use indecisive_compactor::*;
 pub use remove_duplicates::*;
 pub use tombstone_compactor::*;
 pub use cut_off_compactor::*;
-pub use sig_compactor::*;
\ No newline at end of file
+pub use sig_compactor::*;
+pub use public_key_compactor::*;
\ No newline at end of file
