@@ -1,7 +1,9 @@
 #[allow(unused_imports)]
-use log::{info, error, debug};
+use tracing::{info, warn, debug, error, trace, instrument, span, Level};
 use serde::{Serialize, Deserialize};
 use rand::{RngCore};
+use crate::utils::vec_serialize;
+use crate::utils::vec_deserialize;
 
 use super::*;
 
@@ -12,6 +14,7 @@ use super::*;
 #[derive(Serialize, Deserialize, Debug, Clone, Hash, PartialEq, Eq, PartialOrd, Ord)]
 pub struct InitializationVector
 {
+    #[serde(serialize_with = "vec_serialize", deserialize_with = "vec_deserialize")]
     pub bytes: Vec<u8>,
 }
 
@@ -24,14 +27,38 @@ impl InitializationVector {
         rng.fill_bytes(&mut iv.bytes);
         iv
     }
+}
 
-    pub fn from_bytes(bytes: Vec<u8>) -> InitializationVector {
+impl From<Vec<u8>>
+for InitializationVector
+{
+    fn from(bytes: Vec<u8>) -> InitializationVector {
         InitializationVector {
             bytes,
         }
     }
 }
 
+impl From<&[u8]>
+for InitializationVector
+{
+    fn from(bytes: &[u8]) -> InitializationVector {
+        InitializationVector {
+            bytes: bytes.to_vec(),
+        }
+    }
+}
+
+impl From<&[u8; 16]>
+for InitializationVector
+{
+    fn from(bytes: &[u8; 16]) -> InitializationVector {
+        InitializationVector {
+            bytes: bytes.to_vec(),
+        }
+    }
+}
+
 impl std::fmt::Display
 for InitializationVector
 {
