@@ -1,8 +1,5 @@
 #[allow(unused_imports)]
-use log::{info, error, debug};
-use std::error::Error;
-
-extern crate rmp_serde as rmps;
+use tracing::{info, warn, debug, error, trace, instrument, span, Level};
 
 use super::*;
 
@@ -45,7 +42,4 @@ for ProcessError {
 impl std::error::Error
 for ProcessError
 {
-    fn source(&self) -> Option<&(dyn Error + 'static)> {
-        None
-    }
 }
\ No newline at end of file
