@@ -1,5 +1,5 @@
 #![allow(unused_imports)]
-use log::{error, info, debug};
+use tracing::{error, info, debug};
 use async_trait::async_trait;
 use std::sync::Arc;
 
@@ -9,9 +9,12 @@ use super::OpenAction;
 use super::OpenFlow;
 use crate::chain::ChainKey;
 use crate::conf::ChainBuilder;
-use crate::repository::ChainRepository;
-use crate::error::ChainCreationError;
-use crate::trust::IntegrityMode;
+use crate::error::*;
+use crate::spec::*;
+use crate::chain::Chain;
+use crate::crypto::EncryptKey;
+use crate::crypto::KeySize;
+use crate::crypto::AteHash;
 
 pub struct OpenStaticBuilder
 {
@@ -38,10 +41,14 @@ impl OpenStaticBuilder
         OpenStaticBuilder::new(false, false, None)
     }
 
-    pub async fn all_ethereal() -> OpenStaticBuilder {
+    pub async fn all_ethereal_centralized() -> OpenStaticBuilder {
         OpenStaticBuilder::new(true, true, None)
     }
 
+    pub async fn all_ethereal_distributed() -> OpenStaticBuilder {
+        OpenStaticBuilder::new(true, false, None)
+    }
+
     pub async fn all_persistent_and_centralized_with_root_key(root_key: PublicSignKey) -> OpenStaticBuilder {
         OpenStaticBuilder::new(false, true, Some(root_key))
     }
@@ -50,35 +57,48 @@ impl OpenStaticBuilder
         OpenStaticBuilder::new(false, false, Some(root_key))
     }
 
-    pub async fn all_ethereal_with_root_key(root_key: PublicSignKey) -> OpenStaticBuilder {
+    pub async fn all_ethereal_centralized_with_root_key(root_key: PublicSignKey) -> OpenStaticBuilder {
         OpenStaticBuilder::new(true, true, Some(root_key))
     }
+
+    pub async fn all_ethereal_distributed_with_root_key(root_key: PublicSignKey) -> OpenStaticBuilder {
+        OpenStaticBuilder::new(true, false, Some(root_key))
+    }
 }
 
 #[async_trait]
 impl OpenFlow
 for OpenStaticBuilder
 {
-    async fn open(&self, mut builder: ChainBuilder, key: &ChainKey) -> Result<OpenAction, ChainCreationError> {
+    fn hello_path(&self) -> &str {
+        "/"
+    }
+
+    async fn message_of_the_day(&self, _chain: &Arc<Chain>) -> Result<Option<String>, ChainCreationError> {
+        Ok(None)
+    }
+
+    async fn open(&self, mut builder: ChainBuilder, key: &ChainKey, _wire_encryption: Option<KeySize>) -> Result<OpenAction, ChainCreationError> {
         debug!("open_static: {}", key.to_string());
 
         if let Some(root_key) = &self.root_key {
             builder = builder.add_root_public_key(root_key);
         }
-        builder = builder.integrity(match &self.centralized_integrity {
+        Ok(match self.centralized_integrity {
             true => {
                 debug!("chain-builder: centralized integrity");
-                IntegrityMode::Centralized
+                OpenAction::CentralizedChain
+                {
+                    chain: builder.temporal(self.temporal).build().open(&key).await?,
+                }
             },
             false => {
                 debug!("chain-builder: distributed integrity");
-                IntegrityMode::Distributed
-            }
-        });
-
-        Ok(match &self.centralized_integrity {
-            true => OpenAction::CentralizedChain(builder.temporal(self.temporal).build().open_by_key(&key).await?),
-            false => OpenAction::DistributedChain(builder.temporal(self.temporal).build().open_by_key(&key).await?),
+                OpenAction::DistributedChain
+                {
+                    chain: builder.temporal(self.temporal).build().open(&key).await?,
+                }
+            },
         })
     }
 }
\ No newline at end of file
