@@ -1,8 +1,10 @@
 #![allow(unused_imports)]
 use async_trait::async_trait;
 pub mod basic;
-use crate::{crypto::EncryptKey, session::AteSession};
+use crate::{crypto::EncryptKey, session::AteSessionUser};
 
+use crate::crypto::KeySize;
+use super::crypto::PrivateSignKey;
 use super::crypto::PublicSignKey;
 use super::chain::Chain;
 use super::chain::ChainKey;
@@ -10,46 +12,34 @@ use super::conf::ConfAte;
 use super::conf::ChainBuilder;
 use super::error::ChainCreationError;
 use std::sync::Arc;
+use super::spec::TrustMode;
 
-#[derive(Debug, Clone, Copy, PartialEq, Eq, PartialOrd, Ord)]
-pub enum TrustMode
-{
-    Distributed,
-    Centralized
-}
-
-impl std::str::FromStr
-for TrustMode
-{
-    type Err = &'static str;
-
-    fn from_str(s: &str) -> Result<Self, Self::Err> {
-        match s {
-            "distributed" => Ok(TrustMode::Distributed),
-            "centralized" => Ok(TrustMode::Centralized),
-            _ => Err("valid values are 'distributed', 'centralized'"),
-        }
-    }
-}
+pub type MessageOfTheDay = Option<String>;
 
 pub enum OpenAction
 {
-    /// The open request will be denied
-    Deny(String),
+    /// The open request will be denied (with the following reason)
+    Deny {
+        reason: String
+    },
     /// The open action has resulted in a chain that can be consumed as a distributed chain
     /// (distributed chains can be validated without the need for a central authority as the
     ///  signatures are cryptographically signed)
-    DistributedChain(Arc<Chain>),
+    DistributedChain {
+        chain: Arc<Chain>,
+    },
     /// The open action has resulted in a chain that can be consumed as a centralized chain
     /// (centralized chains are higher performance as signatures are not needed to verify the
     ///  integrity of the tree however it requires the clients to trust the integrity checks
     ///  of the server they are connecting to)
-    CentralizedChain(Arc<Chain>),
+    CentralizedChain {
+        chain: Arc<Chain>,
+    },
     /// The open action has resulted in a private chain that can only be consumed if
     /// the caller has a copy of the encryption key
     PrivateChain {
         chain: Arc<Chain>,
-        session: AteSession
+        session: AteSessionUser,
     },
 }
 
@@ -57,7 +47,11 @@ pub enum OpenAction
 pub trait OpenFlow
 where Self: Send + Sync
 {
-    async fn open(&self, builder: ChainBuilder, key: &ChainKey) -> Result<OpenAction, ChainCreationError>;
+    async fn open(&self, builder: ChainBuilder, key: &ChainKey, wire_encryption: Option<KeySize>) -> Result<OpenAction, ChainCreationError>;
+
+    async fn message_of_the_day(&self, chain: &Arc<Chain>) -> Result<Option<String>, ChainCreationError>;
+
+    fn hello_path(&self) -> &str;
 }
 
 pub async fn all_persistent_and_centralized() -> Box<basic::OpenStaticBuilder> {
@@ -68,8 +62,12 @@ pub async fn all_persistent_and_distributed() -> Box<basic::OpenStaticBuilder> {
     Box::new(basic::OpenStaticBuilder::all_persistent_and_distributed().await)
 }
 
-pub async fn all_ethereal() -> Box<basic::OpenStaticBuilder> {
-    Box::new(basic::OpenStaticBuilder::all_ethereal().await)
+pub async fn all_ethereal_distributed() -> Box<basic::OpenStaticBuilder> {
+    Box::new(basic::OpenStaticBuilder::all_ethereal_distributed().await)
+}
+
+pub async fn all_ethereal_centralized() -> Box<basic::OpenStaticBuilder> {
+    Box::new(basic::OpenStaticBuilder::all_ethereal_centralized().await)
 }
 
 pub async fn all_persistent_and_centralized_with_root_key(root_key: PublicSignKey) -> Box<basic::OpenStaticBuilder> {
@@ -80,6 +78,10 @@ pub async fn all_persistent_and_distributed_with_root_key(root_key: PublicSignKe
     Box::new(basic::OpenStaticBuilder::all_persistent_and_distributed_with_root_key(root_key).await)
 }
 
-pub async fn all_ethereal_with_root_key(root_key: PublicSignKey) -> Box<basic::OpenStaticBuilder> {
-    Box::new(basic::OpenStaticBuilder::all_ethereal_with_root_key(root_key).await)
+pub async fn all_ethereal_centralized_with_root_key(root_key: PublicSignKey) -> Box<basic::OpenStaticBuilder> {
+    Box::new(basic::OpenStaticBuilder::all_ethereal_centralized_with_root_key(root_key).await)
+}
+
+pub async fn all_ethereal_distributed_with_root_key(root_key: PublicSignKey) -> Box<basic::OpenStaticBuilder> {
+    Box::new(basic::OpenStaticBuilder::all_ethereal_distributed_with_root_key(root_key).await)
 }
\ No newline at end of file
