@@ -1,20 +1,11 @@
 #![cfg_attr(not(debug_assertions), allow(dead_code, unused_imports, unused_variables))]
-
-/// You can change the hashing routine with these features
-/// - feature = "use_blake3"
-/// - feature = "use_sha3"
+#![warn(unused_extern_crates)]
 
 /// You can change the log file format with these features
 /// - feature = "use_version1"
 /// - feature = "use_version2"
 
-pub const HASH_ROUTINE: crypto::HashRoutine = if cfg!(feature = "use_blake3") {
-    crypto::HashRoutine::Blake3
-} else if cfg!(feature = "use_sha3") {
-    crypto::HashRoutine::Sha3
-} else {
-    crypto::HashRoutine::Blake3
-};
+pub const HASH_ROUTINE: crypto::HashRoutine = crypto::HashRoutine::Blake3;
 
 pub const LOG_VERSION: spec::EventVersion = spec::EventVersion::V2;
 
@@ -51,5 +42,10 @@ pub mod service;
 pub mod pipe;
 pub mod prelude;
 pub mod anti_replay;
+#[cfg(feature = "enable_server")]
 pub mod flow;
-pub mod repository;
\ No newline at end of file
+pub mod engine;
+#[cfg(feature="enable_dns")]
+pub mod dns;
+
+pub use utils::log_init;
\ No newline at end of file
