@@ -1,4 +1,6 @@
 use async_trait::async_trait;
+#[allow(unused_imports)]
+use tracing::{info, warn, debug, error, trace, instrument, span, Level};
 use tokio::sync::mpsc;
 
 use crate::event::*;
@@ -14,11 +16,14 @@ pub struct LoadData
 }
 
 #[async_trait]
-pub trait Loader: Send + Sync + 'static
+pub trait Loader: Send + Sync
 {
     /// Function invoked when the start of the history is being loaded
     async fn start_of_history(&mut self, _size: usize) { }
 
+    /// Human message sent from the server to this process
+    fn human_message(&mut self, _message: String) { }
+
     /// Events are being processed
     fn feed_events(&mut self, _evts: &Vec<EventData>) { }
 
@@ -62,6 +67,13 @@ for CompositionLoader
         }
     }
 
+    fn human_message(&mut self, message: String)
+    {
+        for loader in self.loaders.iter_mut() {
+            loader.human_message(message.clone());
+        }
+    }
+
     fn feed_events(&mut self, evts: &Vec<EventData>)
     {
         for loader in self.loaders.iter_mut() {
@@ -90,7 +102,7 @@ for CompositionLoader
             err = match loader.failed(err).await {
                 Some(a) => a,
                 None => {
-                    ChainCreationError::InternalError(err_msg.clone())
+                    ChainCreationErrorKind::InternalError(err_msg.clone()).into()
                 }
             };
         }
@@ -126,11 +138,13 @@ for NotificationLoader
 {
     async fn start_of_history(&mut self, _size: usize)
     {
+        trace!("sending notify");
         let _ = self.notify.send(Ok(())).await;
     }
 
     async fn end_of_history(&mut self)
     {
+        trace!("sending notify");
         let _ = self.notify.send(Ok(())).await;
     }
 
