@@ -1,5 +1,5 @@
 use async_trait::async_trait;
-use log::{info, warn, debug, error};
+use tracing::{info, warn, debug, error, trace, instrument, span, Level};
 use serde::{Serialize, Deserialize};
 use std::{collections::BTreeMap, sync::Arc};
 use crate::{header::PrimaryKey, meta::Metadata, pipe::EventPipe};
@@ -17,9 +17,12 @@ use crate::conf::*;
 use crate::mesh::msg::*;
 use crate::mesh::MeshSession;
 use crate::comms::PacketData;
-use crate::spec::SerializationFormat;
+use crate::spec::*;
 use crate::redo::LogLookup;
 use crate::time::ChainTimestamp;
+use crate::comms::StreamTx;
+use crate::comms::StreamTxChannel;
+use crate::comms::Tx;
 
 // Determines how the file-system will react while it is nominal and when it is
 // recovering from a communication failure (valid options are 'async', 'readonly-async',
@@ -75,6 +78,15 @@ impl RecoveryMode
             RecoveryMode::ReadOnlySync => true
         }
     }
+
+    pub fn is_meta_sync(&self) -> bool {
+        match self {
+            RecoveryMode::Async => false,
+            RecoveryMode::Sync => true,
+            RecoveryMode::ReadOnlyAsync => true,
+            RecoveryMode::ReadOnlySync => true
+        }
+    }
 }
 
 impl std::str::FromStr
@@ -93,6 +105,57 @@ for RecoveryMode
     }
 }
 
+// Determines how the redo-log engine will perform its backup and restoration
+// actions. Backup and restoration is required for expansions of the cluster
+// and for storage capacity management.
+#[derive(Debug, Clone, Copy, PartialEq, Eq, PartialOrd, Ord)]
+pub enum BackupMode
+{
+    // No backups or restorations will take place fro this set of redo logs. Using this
+    // mode does not improve performance but can save disk space and simplify the
+    // deployment model. This comes at the price of essentially having no backups.
+    None,
+    // The system will not automatically backup data but it will restore data files from
+    // the backup store before creating new empty log files. This is ideal for migration
+    // environments or replicas.
+    Restore,
+    // Backups will be made whenever the log files rotate and when the system loads then
+    // the restoration folder will be checked before it brings online any backups.
+    Rotating,
+    // ATE will automatically backup data to the backup location whenever the log files
+    // rotate or the process shuts down. Upon bringing online a new chain-of-trust the
+    // backup files will be checked first before starting a new log-file thus providing
+    // an automatic migration and restoration system.
+    Full,
+}
+
+impl std::str::FromStr
+for BackupMode
+{
+    type Err = &'static str;
+
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        match s {
+            "off" => Ok(BackupMode::None),
+            "none" => Ok(BackupMode::None),
+            "restore" => Ok(BackupMode::Restore),
+            "rotating" => Ok(BackupMode::Rotating),
+            "full" => Ok(BackupMode::Full),
+            "auto" => Ok(BackupMode::Full),
+            "on" => Ok(BackupMode::Full),
+            _ => Err("valid values are 'none', 'restore', 'rotating' and 'full'"),
+        }
+    }
+}
+
+/// Result of opening a chain-of-trust
+pub struct OpenedChain
+{
+    pub chain: Arc<Chain>,
+    pub integrity: TrustMode,
+    pub message_of_the_day: Option<String>,
+}
+
 #[derive(Default)]
 pub struct MeshHashTable
 {
@@ -102,7 +165,7 @@ pub struct MeshHashTable
 
 impl MeshHashTable
 {
-    pub(crate) fn lookup(&self, key: &ChainKey) -> Option<MeshAddress> {
+    pub(crate) fn lookup(&self, key: &ChainKey) -> Option<(MeshAddress, u32)> {
         let hash = key.hash();
 
         let mut pointer: Option<usize> = None;
@@ -124,12 +187,42 @@ impl MeshHashTable
         if let Some(a) = pointer {
             let index = a % self.address_lookup.len();
             if let Some(a) = self.address_lookup.get(index) {
-                return Some(a.clone());
+                return Some((a.clone(), index as u32));
             }
         }
         
-        self.address_lookup.iter().map(|a| a.clone()).next()
+        None
     }
+
+    pub(crate) fn derive_id(&self, addr: &MeshAddress) -> Option<u32> {
+        let mut n = 0usize;
+        while n < self.address_lookup.len() {
+            let test = &self.address_lookup[n];
+
+            #[cfg(feature="enable_dns")]
+            match test.host.is_loopback() {
+                true if test.port == addr.port => {
+                    if addr.host.is_loopback() || addr.host.is_unspecified() {
+                        return Some(n as u32);
+                    }
+                },
+                _ => {
+                    if *test == *addr {
+                        return Some(n as u32);
+                    }
+                }
+            }
+
+            #[cfg(not(feature="enable_dns"))]
+            if *test == *addr {
+                return Some(n as u32);
+            }
+
+            n = n + 1;
+        }
+        None
+    }
+
     #[allow(dead_code)]
     pub(crate) fn new(cfg_mesh: &ConfMesh) -> MeshHashTable
     {
@@ -152,8 +245,8 @@ impl MeshHashTable
 async fn stream_events<R>(
     chain: &Arc<Chain>,
     range: R,
-    send_to: &mpsc::Sender<PacketData>,
-    wire_format: SerializationFormat,
+    tx: &mut Tx,
+    strip_signatures: bool,
 )
 -> Result<(), CommsError>
 where R: RangeBounds<ChainTimestamp>
@@ -220,8 +313,13 @@ where R: RangeBounds<ChainTimestamp>
 
         let mut evts = Vec::new();
         for evt in multi.load_many(leafs).await? {
+            let mut meta = evt.data.meta.clone();
+            if strip_signatures {
+                meta.strip_signatures();
+            }
+            
             let evt = MessageEvent {
-                meta: evt.data.meta.clone(),
+                meta,
                 data: match evt.data.data_bytes {
                     Some(a) => Some(a.to_vec()),
                     None => None,
@@ -231,8 +329,8 @@ where R: RangeBounds<ChainTimestamp>
             evts.push(evt);
         }
 
-        debug!("sending {} events", evts.len());
-        PacketData::reply_at(Some(&send_to), wire_format, Message::Events {
+        trace!("sending {} events", evts.len());
+        tx.send_reply_msg(Message::Events {
             commit: None,
             evts
         }).await?;
@@ -242,14 +340,14 @@ where R: RangeBounds<ChainTimestamp>
 pub(super) async fn stream_empty_history(
     chain: Arc<Chain>,
     to: Option<ChainTimestamp>,
-    reply_at: mpsc::Sender<PacketData>,
+    tx: &mut StreamTxChannel,
     wire_format: SerializationFormat,
 )
 -> Result<(), CommsError>
 {
     // Extract the root keys and integrity mode
     let (integrity, root_keys) = {
-        let chain = chain.inside_sync.read();
+        let chain = chain.inside_sync.read().unwrap();
         let root_keys = chain
             .plugins
             .iter()
@@ -259,8 +357,8 @@ pub(super) async fn stream_empty_history(
     };
 
     // Let the caller know we will be streaming them events
-    debug!("sending start-of-history (size={})", 0);
-    PacketData::reply_at(Some(&reply_at), wire_format,Message::StartOfHistory
+    trace!("sending start-of-history (size={})", 0);
+    PacketData::reply_at(tx, wire_format,Message::StartOfHistory
         {
             size: 0,
             from: None,
@@ -270,23 +368,23 @@ pub(super) async fn stream_empty_history(
         }).await?;
 
     // Let caller know we have sent all the events that were requested
-    debug!("sending end-of-history");
-    PacketData::reply_at(Some(&reply_at), wire_format, Message::EndOfHistory).await?;
+    trace!("sending end-of-history");
+    PacketData::reply_at(tx, wire_format, Message::EndOfHistory).await?;
     Ok(())
 }
 
 pub(super) async fn stream_history_range<R>(
     chain: Arc<Chain>,
     range: R,
-    reply_at: mpsc::Sender<PacketData>,
-    wire_format: SerializationFormat,
+    tx: &mut Tx,
+    strip_signatures: bool,
 )
 -> Result<(), CommsError>
-where R: RangeBounds<ChainTimestamp>
+where R: RangeBounds<ChainTimestamp>,
 {
     // Extract the root keys and integrity mode
     let (integrity, root_keys) = {
-        let chain = chain.inside_sync.read();
+        let chain = chain.inside_sync.read().unwrap();
         let root_keys = chain
             .plugins
             .iter()
@@ -303,9 +401,8 @@ where R: RangeBounds<ChainTimestamp>
     };
 
     // Let the caller know we will be streaming them events
-    debug!("sending start-of-history (size={})", size);
-    PacketData::reply_at(Some(&reply_at), wire_format,
-    Message::StartOfHistory
+    trace!("sending start-of-history (size={})", size);
+    tx.send_reply_msg(Message::StartOfHistory
         {
             size,
             from: match range.start_bound() {
@@ -317,19 +414,19 @@ where R: RangeBounds<ChainTimestamp>
                 Bound::Included(a) | Bound::Excluded(a) => Some(a.clone())
             },
             root_keys,
-            integrity,
+            integrity: integrity.as_client(),
         }).await?;
 
     // Only if there are things to send
     if size > 0
     {
         // Sync the events
-        debug!("streaming requested events");
-        stream_events(&chain, range, &reply_at, wire_format).await?;
+        trace!("streaming requested events");
+        stream_events(&chain, range, tx, strip_signatures).await?;
     }
 
     // Let caller know we have sent all the events that were requested
-    debug!("sending end-of-history");
-    PacketData::reply_at(Some(&reply_at), wire_format, Message::EndOfHistory).await?;
+    trace!("sending end-of-history");
+    tx.send_reply_msg(Message::EndOfHistory).await?;
     Ok(())
 }
\ No newline at end of file
