@@ -12,11 +12,12 @@ use crate::chain::Chain;
 use crate::error::*;
 use crate::header::PrimaryKey;
 use crate::spec::*;
-use crate::session::AteSession;
+use crate::session::AteSessionUser;
 use crate::crypto::PublicSignKey;
-use crate::trust::IntegrityMode;
 use crate::time::ChainTimestamp;
 
+use super::NodeId;
+
 #[derive(Serialize, Deserialize, Debug, Clone)]
 pub(super) struct MessageEvent
 {
@@ -74,19 +75,63 @@ impl MessageEvent
 }
 
 #[derive(Serialize, Deserialize, Debug, Clone)]
-pub(super) enum Message {
-    Noop,
-    Connected,
-    Disconnected,
-
-    Subscribe {
-        chain_key: ChainKey,
-        from: ChainTimestamp
-    },
-    
+pub enum FatalTerminate
+{
     NotYetSubscribed,
     NotFound,
     NotThisRoot,
+    RootRedirect {
+        expected: u32,
+        actual: u32,
+    },
+    Denied {
+        reason: String
+    },
+    Other {
+        err: String
+    },
+}
+
+impl std::fmt::Display
+for FatalTerminate {
+    fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
+        match self {
+            FatalTerminate::NotYetSubscribed => {
+                write!(f, "Performed an action while the chain is not subscribed")
+            },
+            FatalTerminate::NotFound => {
+                write!(f, "The chain is not found")
+            },
+            FatalTerminate::NotThisRoot => {
+                write!(f, "Failed to create chain-of-trust as this is the wrong root node")
+            },
+            FatalTerminate::RootRedirect { expected, actual } => {
+                write!(f, "Failed to create chain-of-trust as the server you connected (node_id={}) is not hosting these chains - instead you must connect to another node (node_id={})", actual, expected)
+            },
+            FatalTerminate::Denied { reason } => {
+                write!(f, "Access to this chain is denied - {}", reason)
+            },
+            FatalTerminate::Other { err } => {
+                write!(f, "Fatal error occured - {}", err)
+            },
+        }
+    }
+}
+
+#[derive(Serialize, Deserialize, Debug, Clone)]
+pub(super) enum Message {
+    Noop,
+
+    Subscribe {
+        chain_key: ChainKey,
+        from: ChainTimestamp,
+        allow_redirect: bool,
+    },
+
+    HumanMessage {
+        message: String,
+    },
+    ReadOnly,
 
     Lock {
         key: PrimaryKey,
@@ -99,11 +144,15 @@ pub(super) enum Message {
         is_locked: bool
     },
 
+    NewConversation {
+        conversation_id: AteHash,
+    },
+
     StartOfHistory {
         size: usize,
         from: Option<ChainTimestamp>,
         to: Option<ChainTimestamp>,
-        integrity: IntegrityMode,
+        integrity: TrustMode,
         root_keys: Vec<PublicSignKey>,
     },
     Events {
@@ -119,11 +168,9 @@ pub(super) enum Message {
         err: String,
     },
 
-    FatalTerminate {
-        err: String
-    },
+    FatalTerminate(FatalTerminate),
 
-    SecuredWith(AteSession),
+    SecuredWith(AteSessionUser),
 }
 
 impl Default
