@@ -1,9 +1,8 @@
 use tokio::sync::RwLock;
-use parking_lot::RwLock as StdRwLock;
-#[allow(unused_imports)]
-use std::sync::mpsc as smpsc;
+use std::sync::RwLock as StdRwLock;
 #[allow(unused_imports)]
 use std::sync::{Weak, Arc};
+use std::time::Duration;
 
 use crate::session::{AteSession};
 
@@ -16,7 +15,6 @@ use super::header::*;
 use super::lint::*;
 use super::index::*;
 use super::transaction::*;
-use super::repository::*;
 use super::spec::MessageFormat;
 
 use bytes::Bytes;
@@ -24,7 +22,7 @@ use bytes::Bytes;
 pub(crate) struct ChainMultiUserLock<'a>
 {
     pub inside_async: tokio::sync::RwLockReadGuard<'a, ChainProtectedAsync>,
-    pub inside_sync: parking_lot::RwLockReadGuard<'a, ChainProtectedSync>,
+    pub inside_sync: std::sync::RwLockReadGuard<'a, ChainProtectedSync>,
 }
 
 impl<'a> std::ops::Deref
@@ -39,9 +37,7 @@ for ChainMultiUserLock<'a>
 
 #[derive(Clone)]
 pub struct ChainMultiUser
-where Self: Send + Sync
 {
-    pub(super) chain: ChainKey,
     pub(super) inside_async: Arc<RwLock<ChainProtectedAsync>>,
     pub(super) inside_sync: Arc<StdRwLock<ChainProtectedSync>>,
     pub(super) pipe: Arc<Box<dyn EventPipe>>,
@@ -50,14 +46,13 @@ where Self: Send + Sync
 
 impl ChainMultiUser
 {
-    pub(crate) async fn new(accessor: &Chain) -> ChainMultiUser
+    pub(crate) async fn new(chain: &Chain) -> ChainMultiUser
     {
         ChainMultiUser {
-            chain: accessor.key().clone(),
-            inside_async: Arc::clone(&accessor.inside_async),
-            inside_sync: Arc::clone(&accessor.inside_sync),
-            pipe: Arc::clone(&accessor.pipe),
-            default_format: accessor.default_format
+            inside_async: Arc::clone(&chain.inside_async),
+            inside_sync: Arc::clone(&chain.inside_sync),
+            pipe: Arc::clone(&chain.pipe),
+            default_format: chain.default_format
         }
     }
 
@@ -65,7 +60,6 @@ impl ChainMultiUser
     {
         let guard = inside_async.read().await;
         ChainMultiUser {
-            chain: guard.chain.key.clone(),
             inside_async: Arc::clone(inside_async),
             inside_sync: Arc::clone(inside_sync),
             pipe: Arc::clone(pipe),
@@ -98,26 +92,26 @@ impl ChainMultiUser
     }
 
     #[allow(dead_code)]
-    pub(crate) fn metadata_lint_many<'a>(&self, lints: &Vec<LintData<'a>>, session: &AteSession, conversation: Option<&Arc<ConversationSession>>) -> Result<Vec<CoreMetadata>, LintError> {
-        let guard = self.inside_sync.read();
+    pub(crate) fn metadata_lint_many<'a>(&self, lints: &Vec<LintData<'a>>, session: &'_ dyn AteSession, conversation: Option<&Arc<ConversationSession>>) -> Result<Vec<CoreMetadata>, LintError> {
+        let guard = self.inside_sync.read().unwrap();
         guard.metadata_lint_many(lints, session, conversation)
     }
 
     #[allow(dead_code)]
-    pub(crate) fn metadata_lint_event(&self, meta: &mut Metadata, session: &AteSession, trans_meta: &TransactionMetadata) -> Result<Vec<CoreMetadata>, LintError> {
-        let guard = self.inside_sync.read();
-        guard.metadata_lint_event(meta, session, trans_meta)
+    pub(crate) fn metadata_lint_event(&self, meta: &mut Metadata, session: &'_ dyn AteSession, trans_meta: &TransactionMetadata, type_code: &str) -> Result<Vec<CoreMetadata>, LintError> {
+        let guard = self.inside_sync.read().unwrap();
+        guard.metadata_lint_event(meta, session, trans_meta, type_code)
     }
 
     #[allow(dead_code)]
-    pub(crate) fn data_as_overlay(&self, meta: &Metadata, data: Bytes, session: &AteSession) -> Result<Bytes, TransformError> {
-        let guard = self.inside_sync.read();
+    pub(crate) fn data_as_overlay(&self, meta: &Metadata, data: Bytes, session: &'_ dyn AteSession) -> Result<Bytes, TransformError> {
+        let guard = self.inside_sync.read().unwrap();
         guard.data_as_overlay(meta, data, session)
     }
 
     #[allow(dead_code)]
-    pub(crate) fn data_as_underlay(&self, meta: &mut Metadata, data: Bytes, session: &AteSession, trans_meta: &TransactionMetadata) -> Result<Bytes, TransformError> {
-        let guard = self.inside_sync.read();
+    pub(crate) fn data_as_underlay(&self, meta: &mut Metadata, data: Bytes, session: &'_ dyn AteSession, trans_meta: &TransactionMetadata) -> Result<Bytes, TransformError> {
+        let guard = self.inside_sync.read().unwrap();
         guard.data_as_underlay(meta, data, session, trans_meta)
     }
     
@@ -125,35 +119,42 @@ impl ChainMultiUser
         self.inside_async.read().await.chain.redo.count()
     }
 
-    pub fn repository(&self) -> Option<Arc<dyn ChainRepository>> {
-        self.inside_sync.read().repository()
-    }
-
     pub(crate) async fn lock<'a>(&'a self) -> ChainMultiUserLock<'a> {
         ChainMultiUserLock {
             inside_async: self.inside_async.read().await,
-            inside_sync: self.inside_sync.read()
+            inside_sync: self.inside_sync.read().unwrap()
         }        
     }
 
     pub async fn sync(&self) -> Result<(), CommitError>
+    {
+        let timeout = Duration::from_secs(30);
+        self.sync_ext(timeout).await
+    }
+
+    pub async fn sync_ext(&self, timeout: Duration) -> Result<(), CommitError>
     {
         // Create the transaction
         let trans = Transaction {
             scope: TransactionScope::Full,
             transmit: true,
             events: Vec::new(),
+            timeout,
             conversation: None,
         };
 
+        let work = ChainWork {
+            trans,
+        };
+
         // Process the transaction in the chain using its pipe
-        self.pipe.feed(trans).await?;
+        self.pipe.feed(work).await?;
         Ok(())
     }
 }
 
 impl ChainProtectedSync {
-    pub(crate) fn metadata_lint_many<'a>(&self, lints: &Vec<LintData<'a>>, session: &AteSession, conversation: Option<&Arc<ConversationSession>>) -> Result<Vec<CoreMetadata>, LintError> {
+    pub(crate) fn metadata_lint_many<'a>(&self, lints: &Vec<LintData<'a>>, session: &'_ dyn AteSession, conversation: Option<&Arc<ConversationSession>>) -> Result<Vec<CoreMetadata>, LintError> {
         let mut ret = Vec::new();
         for linter in self.linters.iter() {
             ret.extend(linter.metadata_lint_many(lints, session, conversation)?);
@@ -164,18 +165,18 @@ impl ChainProtectedSync {
         Ok(ret)
     }
 
-    pub(crate) fn metadata_lint_event(&self, meta: &mut Metadata, session: &AteSession, trans_meta: &TransactionMetadata) -> Result<Vec<CoreMetadata>, LintError> {
+    pub(crate) fn metadata_lint_event(&self, meta: &mut Metadata, session: &'_ dyn AteSession, trans_meta: &TransactionMetadata, type_code: &str) -> Result<Vec<CoreMetadata>, LintError> {
         let mut ret = Vec::new();
         for linter in self.linters.iter() {
-            ret.extend(linter.metadata_lint_event(meta, session, trans_meta)?);
+            ret.extend(linter.metadata_lint_event(meta, session, trans_meta, type_code)?);
         }
         for plugin in self.plugins.iter() {
-            ret.extend(plugin.metadata_lint_event(meta, session, trans_meta)?);
+            ret.extend(plugin.metadata_lint_event(meta, session, trans_meta, type_code)?);
         }
         Ok(ret)
     }
 
-    pub(crate) fn data_as_overlay(&self, meta: &Metadata, data: Bytes, session: &AteSession) -> Result<Bytes, TransformError> {
+    pub(crate) fn data_as_overlay(&self, meta: &Metadata, data: Bytes, session: &'_ dyn AteSession) -> Result<Bytes, TransformError> {
         let mut ret = data;
         for plugin in self.plugins.iter().rev() {
             ret = plugin.data_as_overlay(meta, ret, session)?;
@@ -186,7 +187,7 @@ impl ChainProtectedSync {
         Ok(ret)
     }
 
-    pub(crate) fn data_as_underlay(&self, meta: &mut Metadata, data: Bytes, session: &AteSession, trans_meta: &TransactionMetadata) -> Result<Bytes, TransformError> {
+    pub(crate) fn data_as_underlay(&self, meta: &mut Metadata, data: Bytes, session: &'_ dyn AteSession, trans_meta: &TransactionMetadata) -> Result<Bytes, TransformError> {
         let mut ret = data;
         for transformer in self.transformers.iter() {
             ret = transformer.data_as_underlay(meta, ret, session, trans_meta)?;
@@ -196,11 +197,4 @@ impl ChainProtectedSync {
         }
         Ok(ret)
     }
-    
-    pub fn repository(&self) -> Option<Arc<dyn ChainRepository>> {
-        match &self.repository {
-            Some(a) => a.upgrade(),
-            None => None
-        }
-    }
 }
\ No newline at end of file
