@@ -1,37 +1,39 @@
-#[allow(unused_imports)]
-use log::{error, info, warn, debug};
+#![allow(unused_imports)]
+use tracing::{error, info, warn, debug, trace};
 use async_trait::async_trait;
-#[cfg(feature = "local_fs")]
+#[cfg(feature = "enable_local_fs")]
 use std::{collections::VecDeque};
 use tokio::io::Result;
 use tokio::io::Error;
 use tokio::io::ErrorKind;
+use std::pin::Pin;
 
 use crate::crypto::*;
-#[cfg(feature = "local_fs")]
+#[cfg(feature = "enable_local_fs")]
 use crate::conf::*;
-#[cfg(feature = "local_fs")]
+#[cfg(feature = "enable_local_fs")]
 use crate::chain::*;
 use crate::event::*;
 use crate::error::*;
 use crate::loader::*;
 use crate::redo::LogLookup;
+use crate::mesh::BackupMode;
 
 use super::*;
 use super::flip::RedoLogFlip;
-#[cfg(feature = "local_fs")]
+#[cfg(feature = "enable_local_fs")]
 use super::flags::OpenFlags;
 use super::flip::FlippedLogFile;
-#[cfg(feature = "local_fs")]
+#[cfg(feature = "enable_local_fs")]
 use super::loader::RedoLogLoader;
 use super::api::LogWritable;
-#[cfg(feature = "local_fs")]
-use super::file_localfs::LogFileLocalFs;
-use super::file_memdb::LogFileMemDb;
+#[cfg(feature = "enable_local_fs")]
+use super::log_localfs::LogFileLocalFs;
+use super::log_memdb::LogFileMemDb;
 
 pub struct RedoLog
 {
-    #[cfg(feature = "local_fs")]
+    #[cfg(feature = "enable_local_fs")]
     log_path: Option<String>,
     flip: Option<RedoLogFlip>,
     pub(super) log_file: Box<dyn LogFile>,
@@ -39,8 +41,8 @@ pub struct RedoLog
 
 impl RedoLog
 {
-    #[cfg(feature = "local_fs")]
-    async fn new(path_log: Option<String>, flags: OpenFlags, cache_size: usize, cache_ttl: u64, loader: Box<impl Loader>, header_bytes: Vec<u8>) -> std::result::Result<RedoLog, SerializationError>
+    #[cfg(feature = "enable_local_fs")]
+    async fn new(path_log: Option<String>, backup_path: Option<String>, restore_path: Option<String>, flags: OpenFlags, cache_size: usize, cache_ttl: u64, loader: Box<impl Loader>, header_bytes: Vec<u8>) -> std::result::Result<RedoLog, SerializationError>
     {
         // Now load the real thing
         let ret = RedoLog {
@@ -49,7 +51,10 @@ impl RedoLog
                 Some(path_log) => {
                     let mut log_file = LogFileLocalFs::new(
                         flags.temporal,
+                        flags.read_only,
                         path_log,
+                        backup_path,
+                        restore_path,
                         flags.truncate,
                         cache_size,
                         cache_ttl,
@@ -67,7 +72,7 @@ impl RedoLog
         Ok(ret)
     }
 
-    #[cfg(not(feature = "local_fs"))]
+    #[cfg(not(feature = "enable_local_fs"))]
     async fn new(header_bytes: Vec<u8>) -> std::result::Result<RedoLog, SerializationError>
     {
         // Now load the real thing
@@ -78,11 +83,15 @@ impl RedoLog
         Ok(ret)
     }
 
-    #[cfg(feature = "rotate")]
+    #[cfg(feature = "enable_rotate")]
     pub async fn rotate(&mut self, header_bytes: Vec<u8>) -> Result<()> {
         Ok(self.log_file.rotate(header_bytes).await?)
     }
 
+    pub fn backup(&mut self, include_active_files: bool) -> Result<Pin<Box<dyn futures::Future<Output=Result<()>> + Send + Sync>>> {
+        Ok(self.log_file.backup(include_active_files)?)
+    }
+
     pub async fn begin_flip(&mut self, header_bytes: Vec<u8>) -> Result<FlippedLogFile> {
         
         match self.flip
@@ -107,7 +116,7 @@ impl RedoLog
         }
     }
 
-    pub async fn finish_flip(&mut self, mut flip: FlippedLogFile, mut deferred_write_callback: impl FnMut(LogLookup, &EventHeader)) -> std::result::Result<Vec<EventHeaderRaw>, SerializationError>
+    pub async fn finish_flip(&mut self, mut flip: FlippedLogFile, mut deferred_write_callback: impl FnMut(LogLookup, EventHeader)) -> std::result::Result<Vec<EventHeaderRaw>, SerializationError>
     {
         match &mut self.flip
         {
@@ -122,12 +131,12 @@ impl RedoLog
                     event_summary.push(header.raw.clone());
                     let lookup = new_log_file.write(&d).await?;
     
-                    deferred_write_callback(lookup, &header);
+                    deferred_write_callback(lookup, header);
                 }
                 
                 new_log_file.flush().await?;
                 
-                #[cfg(feature = "local_fs")]
+                #[cfg(feature = "enable_local_fs")]
                 if let Some(a) = self.log_path.as_ref() {
                     new_log_file.move_log_file(a)?;
                 }
@@ -139,7 +148,7 @@ impl RedoLog
             },
             None =>
             {
-                Err(SerializationError::IO(Error::new(ErrorKind::Other, "There is no outstanding flip operation to end.")))
+                Err(SerializationErrorKind::IO(Error::new(ErrorKind::Other, "There is no outstanding flip operation to end.")).into())
             }
         }
     }
@@ -167,15 +176,14 @@ impl RedoLog
         }
     }
 
-    #[cfg(feature = "local_fs")]
+    #[cfg(feature = "enable_local_fs")]
     pub async fn open(cfg: &ConfAte, key: &ChainKey, flags: OpenFlags, header_bytes: Vec<u8>) -> std::result::Result<(RedoLog, VecDeque<LoadData>), SerializationError>
     {
-        let mut ret = VecDeque::new();
         let (loader, mut rx) = RedoLogLoader::new();
-
+        
         let cfg = cfg.clone();
         let key = key.clone();
-        let log = tokio::spawn(async move {
+        let join1 = async move {
             RedoLog::open_ext(
                 &cfg,
                 &key,
@@ -183,23 +191,29 @@ impl RedoLog
                 loader,
                 header_bytes
             ).await
-        });
+        };
 
-        while let Some(evt) = rx.recv().await {
-            ret.push_back(evt);
-        }
+        let join2 = async move {
+            let mut ret = VecDeque::new();
+            while let Some(evt) = rx.recv().await {
+                ret.push_back(evt);
+            }
+            ret
+        };
+
+        let (log, ret) = futures::join!(join1, join2);
         
-        let log = log.await.unwrap()?;
-        Ok((log, ret))
+        Ok((log?, ret))
     }
 
-    #[cfg(feature = "local_fs")]
+    #[cfg(feature = "enable_local_fs")]
     pub async fn open_ext(cfg: &ConfAte, key: &ChainKey, flags: OpenFlags, loader: Box<impl Loader>, header_bytes: Vec<u8>) -> std::result::Result<RedoLog, SerializationError> {
         let mut key_name = key.name.clone();
         if key_name.starts_with("/") {
             key_name = key_name[1..].to_string();
         }
 
+        trace!("temporal: {}", flags.temporal);
         let path_log = match flags.temporal
         {
             false => {
@@ -213,13 +227,39 @@ impl RedoLog
         };
         
         if let Some(path_log) = path_log.as_ref() {
+            trace!("log-path: {}", path_log);
             let path = std::path::Path::new(path_log);
             let _ = std::fs::create_dir_all(path.parent().unwrap().clone());
+        } else {
+            trace!("log-path: (memory)");
+        }
+
+        let mut backup_path = {
+            match cfg.backup_path.as_ref() {
+                Some(a) if a.ends_with("/") => Some(format!("{}{}.log", a, key_name)),
+                Some(a) => Some(format!("{}/{}.log", a, key_name)),
+                None => None,
+            }
+        };
+
+        if let Some(backup_path) = backup_path.as_ref() {
+            let path = std::path::Path::new(backup_path);
+            let _ = std::fs::create_dir_all(path.parent().unwrap().clone());
         }
 
+        let mut restore_path = backup_path.clone();
+        match cfg.backup_mode {
+            BackupMode::None => { restore_path = None; backup_path = None; }
+            BackupMode::Restore => { backup_path = None; }
+            BackupMode::Rotating => { }
+            BackupMode::Full => { }
+        };
+        
         let log = {
             RedoLog::new(
                 path_log.clone(),
+                backup_path.clone(),
+                restore_path.clone(),
                 flags,
                 cfg.load_cache_size,
                 cfg.load_cache_ttl,
@@ -231,7 +271,7 @@ impl RedoLog
         Ok(log)
     }
 
-    #[cfg(not(feature = "local_fs"))]
+    #[cfg(not(feature = "enable_local_fs"))]
     pub async fn open(header_bytes: Vec<u8>) -> std::result::Result<RedoLog, SerializationError> {
         let log = {
             RedoLog::new(
