@@ -1,5 +1,5 @@
 #[allow(unused_imports)]
-use log::{error, info, warn, debug};
+use tracing::{error, info, warn, debug};
 use async_trait::async_trait;
 use tokio::io::Result;
 
@@ -7,7 +7,7 @@ use crate::{crypto::AteHash};
 use crate::event::*;
 use crate::error::*;
 
-use super::{LogLookup, file::LogFile};
+use super::{LogLookup, log_traits::LogFile};
 use super::api::LogWritable;
 use super::core::RedoLog;
 
