@@ -1,18 +1,18 @@
 #[allow(unused_imports)]
-use log::{error, info, warn, debug};
+use tracing::{error, info, warn, debug};
 
 mod api;
-#[cfg(feature = "local_fs")]
-mod file_localfs;
-mod file_memdb;
-mod file;
+#[cfg(feature = "enable_local_fs")]
+mod log_localfs;
+mod log_memdb;
+mod log_traits;
 mod flags;
 mod flip;
 mod magic;
-#[cfg(feature = "local_fs")]
+#[cfg(feature = "enable_local_fs")]
 mod appender;
 mod loader;
-#[cfg(feature = "local_fs")]
+#[cfg(feature = "enable_local_fs")]
 mod archive;
 mod core;
 mod test;
@@ -24,4 +24,4 @@ pub use api::LogWritable;
 
 pub(crate) use api::LogLookup;
 
-pub use file::*;
\ No newline at end of file
+pub use log_traits::*;
\ No newline at end of file
