@@ -1,6 +1,6 @@
 #![cfg(test)]
 #[allow(unused_imports)]
-use log::{error, info, warn, debug};
+use tracing::{error, info, warn, debug};
 use tokio::runtime::Runtime;
 use bytes::Bytes;
 
@@ -13,7 +13,7 @@ use crate::spec::*;
 
 use super::api::LogWritable;
 use super::core::RedoLog;
-#[cfg(feature = "local_fs")]
+#[cfg(feature = "enable_local_fs")]
 use super::flags::OpenFlags;
 
 /* 
@@ -70,7 +70,7 @@ async fn test_read_data(log: &mut RedoLog, read_header: AteHash, test_key: Prima
 
 #[test]
 fn test_redo_log() {
-    crate::utils::bootstrap_env();
+    crate::utils::bootstrap_test_env();
 
     let rt = Runtime::new().unwrap();
 
@@ -91,9 +91,9 @@ fn test_redo_log() {
         {
             // Open the log once for writing
             println!("test_redo_log - creating the redo log");
-            #[cfg(feature = "local_fs")]
-            let (mut rl, _) = RedoLog::open(&mock_cfg, &mock_chain_key, OpenFlags::create_centralized(), Vec::new()).await.expect("Failed to load the redo log");
-            #[cfg(not(feature = "local_fs"))]
+            #[cfg(feature = "enable_local_fs")]
+            let (mut rl, _) = RedoLog::open(&mock_cfg, &mock_chain_key, OpenFlags::create_centralized_server(), Vec::new()).await.expect("Failed to load the redo log");
+            #[cfg(not(feature = "enable_local_fs"))]
             let mut rl = RedoLog::open(Vec::new()).await.expect("Failed to load the redo log");
             
             // Test that its empty
@@ -161,12 +161,12 @@ fn test_redo_log() {
         {
             // Open it up again which should check that it loads data properly
             println!("test_redo_log - reopening the redo log");
-            #[cfg(feature = "local_fs")]
-            let (mut rl, mut loader) = RedoLog::open(&mock_cfg, &mock_chain_key, OpenFlags::open_centralized(), Vec::new()).await.expect("Failed to load the redo log");
-            #[cfg(not(feature = "local_fs"))]
+            #[cfg(feature = "enable_local_fs")]
+            let (mut rl, mut loader) = RedoLog::open(&mock_cfg, &mock_chain_key, OpenFlags::open_centralized_server(), Vec::new()).await.expect("Failed to load the redo log");
+            #[cfg(not(feature = "enable_local_fs"))]
             let mut rl = RedoLog::open(Vec::new()).await.expect("Failed to load the redo log");
             
-            #[cfg(feature = "local_fs")]
+            #[cfg(feature = "enable_local_fs")]
             {
                 // Check that the correct data is read
                 println!("test_redo_log - testing read result of blah1 (again)");
@@ -185,16 +185,16 @@ fn test_redo_log() {
             println!("test_redo_log - writing test data to log - blah7");
             let halb7 = test_write_data(&mut rl, blah7, Some(vec![7; 10]), true, mock_cfg.log_format).await;
 
-            #[cfg(feature = "local_fs")]
+            #[cfg(feature = "enable_local_fs")]
             assert_eq!(5, rl.count());
-            #[cfg(not(feature = "local_fs"))]
+            #[cfg(not(feature = "enable_local_fs"))]
             assert_eq!(1, rl.count());
     
             // Read the test data again
             println!("test_redo_log - testing read result of blah7");
             test_read_data(&mut rl, halb7, blah7, Some(vec![7; 10]), mock_cfg.log_format).await;
             println!("test_redo_log - confirming no more data");
-            #[cfg(feature = "local_fs")]
+            #[cfg(feature = "enable_local_fs")]
             assert_eq!(5, rl.count());
 
             rl.destroy().unwrap();
