@@ -1,7 +1,6 @@
 pub mod chain_invoke;
 pub mod chain_sniffer;
 pub mod helper;
-pub mod invocation_context;
 pub mod notify;
 pub mod service_handler;
 pub mod service_hook;
@@ -10,10 +9,9 @@ pub mod tests;
 
 pub(crate) use chain_sniffer::*;
 pub(crate) use notify::*;
-pub(crate) use service_hook::*;
 pub(crate) use helper::*;
 
 pub use chain_invoke::*;
-pub use invocation_context::*;
 pub use service_handler::*;
+pub use service_hook::*;
 pub use service::*;
\ No newline at end of file
