@@ -1,5 +1,5 @@
 #[allow(unused_imports)]
-use log::{info, error, warn, debug};
+use tracing::{info, error, warn, debug};
 use tokio::sync::mpsc;
 use std::sync::Arc;
 
@@ -14,6 +14,15 @@ pub(crate) enum NotifyWho
     Service(Arc<dyn Service>)
 }
 
+impl std::fmt::Debug
+for NotifyWho
+{
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        write!(f, "notify-who:sender")
+    }
+}
+
+#[derive(Debug)]
 pub(crate) struct Notify
 {
     pub(crate) key: PrimaryKey,
@@ -22,10 +31,12 @@ pub(crate) struct Notify
 
 impl Notify
 {
-    pub(crate) async fn notify(self) -> Result<(), ServiceError<()>> {
+    pub(crate) async fn notify(self) -> Result<(), InvokeError> {
         match self.who {
             NotifyWho::Sender(sender) => sender.send(self.key).await?,
-            NotifyWho::Service(service) => service.notify(self.key).await?
+            NotifyWho::Service(service) => {
+                service.notify(self.key).await?
+            }
         }
         Ok(())
     }
