@@ -1,19 +1,25 @@
+#[allow(unused_imports)]
+use tracing::{error, info, warn, debug, trace};
 use std::sync::Arc;
 use multimap::MultiMap;
+use std::ops::Deref;
+use error_chain::bail;
 #[allow(unused_imports)]
 use serde::{Serialize, Deserialize, de::DeserializeOwned};
 use fxhash::FxHashMap;
 #[allow(unused_imports)]
 use crate::crypto::{EncryptedPrivateKey, AteHash, DoubleHash, PublicSignKey};
 #[allow(unused_imports)]
-use crate::session::{AteSession, AteSessionProperty};
+use crate::session::{AteSession, AteSessionProperty, AteSessionKeyCategory};
+use crate::utils::vec_serialize;
+use crate::utils::vec_deserialize;
+use crate::spec::*;
 
 use super::validator::EventValidator;
 use super::lint::EventMetadataLinter;
 use super::transform::EventDataTransformer;
 #[allow(unused_imports)]
 use super::sink::{EventSink};
-use super::trust::IntegrityMode;
 
 use super::event::*;
 use super::error::*;
@@ -26,6 +32,7 @@ use super::transaction::*;
 pub struct MetaSignature
 {
     pub hashes: Vec<AteHash>,
+    #[serde(serialize_with = "vec_serialize", deserialize_with = "vec_deserialize")]
     pub signature: Vec<u8>,
     pub public_key_hash: AteHash,
 }
@@ -75,7 +82,7 @@ pub struct SignaturePlugin
 {
     pk: FxHashMap<AteHash, PublicSignKey>,
     sigs: MultiMap<AteHash, AteHash>,
-    integrity: IntegrityMode,
+    integrity: TrustMode,
 }
 
 impl SignaturePlugin
@@ -85,7 +92,7 @@ impl SignaturePlugin
         SignaturePlugin {
             pk: FxHashMap::default(),
             sigs: MultiMap::default(),
-            integrity: IntegrityMode::Distributed,
+            integrity: TrustMode::Distributed,
         }
     }
 
@@ -124,19 +131,28 @@ for SignaturePlugin
         for m in header.meta.core.iter() {
             match m {
                 CoreMetadata::Signature(sig) => {
-                    let pk = match self.pk.get(&sig.public_key_hash) {
-                        Some(pk) => pk,
-                        None => { return Result::Err(SinkError::MissingPublicKey(sig.public_key_hash)); }
-                    };
+                    if self.integrity == TrustMode::Distributed || self.integrity == TrustMode::Centralized(CentralizedRole::Server) {
+                        let pk = match self.pk.get(&sig.public_key_hash) {
+                            Some(pk) => pk,
+                            None => {
+                                trace!("signature missing public key: {}", sig.public_key_hash);
+                                bail!(SinkErrorKind::MissingPublicKey(sig.public_key_hash))
+                            }
+                        };
 
-                    let hashes_bytes: Vec<u8> = sig.hashes.iter().flat_map(|h| { Vec::from(h.val).into_iter() }).collect();
-                    let hash_of_hashes = AteHash::from_bytes(&hashes_bytes[..]);
-                    let result = match pk.verify(&hash_of_hashes.val[..], &sig.signature[..]) {
-                        Ok(r) => r,
-                        Err(err) => { return Result::Err(SinkError::InvalidSignature { hash: sig.public_key_hash, err: Some(err) }); },
-                    };
-                    if result == false {
-                        return Result::Err(SinkError::InvalidSignature { hash: sig.public_key_hash, err: None });
+                        let hashes_bytes: Vec<u8> = sig.hashes.iter().flat_map(|h| { Vec::from(h.val).into_iter() }).collect();
+                        let hash_of_hashes = AteHash::from_bytes(&hashes_bytes[..]);
+                        let result = match pk.verify(&hash_of_hashes.val[..], &sig.signature[..]) {
+                            Ok(r) => r,
+                            Err(err) => {
+                                trace!("signature is invalid: {}", err);
+                                bail!(SinkErrorKind::InvalidSignature(sig.public_key_hash, Some(err)))
+                            },
+                        };
+                        if result == false {
+                            trace!("signature failed validate - {}", sig.public_key_hash);
+                            bail!(SinkErrorKind::InvalidSignature(sig.public_key_hash, None));
+                        }
                     }
 
                     // Add all the validated hashes
@@ -147,10 +163,14 @@ for SignaturePlugin
                     // If we in a conversation and integrity is centrally managed then update the
                     // conversation so that we record that a signature was validated for a hash
                     // which is clear proof of ownershp
-                    if self.integrity == IntegrityMode::Centralized {
+                    if self.integrity.is_centralized() {
                         if let Some(conversation) = &conversation {
-                            let mut lock = conversation.signatures.write();
-                            lock.insert(sig.public_key_hash);
+                            if let Some(conv_id) = conversation.id.read() {
+                                if sig.hashes.contains(conv_id.deref()) {
+                                    let mut lock = conversation.signatures.write().unwrap();
+                                    lock.insert(sig.public_key_hash);
+                                }
+                            }
                         }
                     }
                 }
@@ -174,7 +194,7 @@ for SignaturePlugin
         Box::new(self.clone())
     }
 
-    fn set_integrity_mode(&mut self, mode: IntegrityMode) {
+    fn set_integrity_mode(&mut self, mode: TrustMode) {
         self.integrity = mode;
     }
 
@@ -190,7 +210,7 @@ for SignaturePlugin
         Box::new(self.clone())
     }
 
-    fn metadata_lint_many<'a>(&self, raw: &Vec<LintData<'a>>, session: &AteSession, conversation: Option<&Arc<ConversationSession>>) -> Result<Vec<CoreMetadata>, LintError>
+    fn metadata_lint_many<'a>(&self, raw: &Vec<LintData<'a>>, session: &'_ dyn AteSession, conversation: Option<&Arc<ConversationSession>>) -> Result<Vec<CoreMetadata>, LintError>
     {
         // If there is no data then we are already done
         let mut ret = Vec::new();
@@ -209,9 +229,9 @@ for SignaturePlugin
 
         // Check the fast path... if we are under centralized integrity and the destination
         // has already got proof that we own the authentication key then we are done
-        if self.integrity == IntegrityMode::Centralized {
+        if self.integrity.is_centralized() {
             if let Some(conversation) = &conversation {
-                let lock = conversation.signatures.read();
+                let lock = conversation.signatures.read().unwrap();
                 auths.retain(|h| lock.contains(h) == false);
             }
         }
@@ -220,16 +240,24 @@ for SignaturePlugin
         for auth in auths.into_iter()
         {
             // Find the session key for it (if one does not exist we have a problem!)
-            let sk = match session.write_keys()
+            let sk = match session.write_keys(AteSessionKeyCategory::AllKeys)
                 .filter(|k| k.hash() == *auth)
                 .next()
             {
                 Some(sk) => sk,
-                None => return Err(LintError::MissingWriteKey(auth.clone())),
+                None => bail!(LintErrorKind::MissingWriteKey(auth.clone())),
             };
 
             // Compute a hash of the hashesevt
             let mut data_hashes = Vec::new();
+            if self.integrity.is_centralized() {
+                if let Some(conversation) = &conversation {
+                    if let Some(conv_id) = conversation.id.read() {
+                        let conv_id = conv_id.deref().clone();
+                        data_hashes.push(conv_id);
+                    }
+                }
+            }
             for e in raw.iter() {
                 if let Some(a) = e.data.meta.get_sign_with() {
                     if a.keys.contains(&auth) == true {
@@ -244,8 +272,8 @@ for SignaturePlugin
             let hash_of_hashes = AteHash::from_bytes(&hashes_bytes[..]);
             
             // Add the public key side into the chain-of-trust if it is not present yet
-            if self.pk.get(&auth).is_none() {
-                ret.push(CoreMetadata::PublicKey(sk.as_public_key()));
+            if self.pk.get(&auth).is_none() || self.integrity.is_centralized() {
+                ret.push(CoreMetadata::PublicKey(sk.as_public_key().clone()));
             };
 
             // Next we need to decrypt the private key and use it to sign the hashes
@@ -261,9 +289,9 @@ for SignaturePlugin
 
             // Save signatures we have sent over this specific conversation so that future
             // transmissions do not need to prove it again (this makes the fast path quicker)
-            if self.integrity == IntegrityMode::Centralized {
+            if self.integrity.is_centralized() {
                 if let Some(conversation) = &conversation {
-                    let mut lock = conversation.signatures.write();
+                    let mut lock = conversation.signatures.write().unwrap();
                     lock.insert((*auth).clone());
                 }
             }
