@@ -35,6 +35,17 @@ for SerializationFormat
 
 impl SerializationFormat
 {
+    pub fn iter() -> std::vec::IntoIter<SerializationFormat>
+    {
+        vec!
+        [
+            SerializationFormat::Json,
+            SerializationFormat::MessagePack,
+            SerializationFormat::Bincode
+        ]
+        .into_iter()
+    }
+
     pub fn serialize<T>(&self, val: &T) -> Result<Vec<u8>, SerializationError>
     where T: Serialize + ?Sized
     {
@@ -86,6 +97,84 @@ pub struct MessageFormat
     pub data: SerializationFormat,
 }
 
+#[derive(Serialize, Deserialize, Debug, Clone, Copy, PartialEq, Eq, PartialOrd, Ord, Hash)]
+pub enum CentralizedRole
+{
+    Server,
+    Client
+}
+
+impl std::fmt::Display
+for CentralizedRole {
+    fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
+        match self {
+            CentralizedRole::Server => write!(f, "server"),
+            CentralizedRole::Client => write!(f, "client")
+        }
+    }
+}
+
+#[derive(Serialize, Deserialize, Debug, Clone, Copy, PartialEq, Eq, PartialOrd, Ord, Hash)]
+pub enum TrustMode
+{
+    Distributed,
+    Centralized(CentralizedRole)
+}
+
+impl TrustMode
+{
+    pub fn is_centralized(&self) -> bool
+    {
+        if let TrustMode::Centralized(_) = self {
+            true
+        } else {
+            false
+        }
+    }
+
+    pub fn as_client(&self) -> TrustMode
+    {
+        match self {
+            TrustMode::Centralized(_) => TrustMode::Centralized(CentralizedRole::Client),
+            a => a.clone()
+        }
+    }
+
+    pub fn as_server(&self) -> TrustMode
+    {
+        match self {
+            TrustMode::Centralized(_) => TrustMode::Centralized(CentralizedRole::Server),
+            a => a.clone()
+        }
+    }
+}
+
+impl std::fmt::Display
+for TrustMode {
+    fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
+        match self {
+            TrustMode::Centralized(a) => write!(f, "centralized({})", a),
+            TrustMode::Distributed => write!(f, "distributed")
+        }
+    }
+}
+
+impl std::str::FromStr
+for TrustMode
+{
+    type Err = &'static str;
+
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        match s {
+            "distributed" => Ok(TrustMode::Distributed),
+            "centralized" => Ok(TrustMode::Centralized(CentralizedRole::Server)),
+            "centralized(client)" => Ok(TrustMode::Centralized(CentralizedRole::Client)),
+            "centralized(server)" => Ok(TrustMode::Centralized(CentralizedRole::Server)),
+            _ => Err("valid values are 'distributed', 'centralized'"),
+        }
+    }
+}
+
 #[derive(Debug, Clone, Copy, Eq, PartialEq, IntoPrimitive, TryFromPrimitive)]
 #[repr(u8)]
 pub enum BlobSize
@@ -159,7 +248,7 @@ impl EventVersion
             Ok(a) => Ok(Some(a)),
             Err(err) => {
                 if err.kind() == ErrorKind::UnexpectedEof { return Ok(None); }
-                Err(SerializationError::IO(tokio::io::Error::new(tokio::io::ErrorKind::Other, format!("Failed to read the event magic number at 0x{:x}", api.offset()))))
+                Err(SerializationErrorKind::IO(tokio::io::Error::new(tokio::io::ErrorKind::Other, format!("Failed to read the event magic number at 0x{:x}", api.offset()))).into())
             },
         }
     }
@@ -203,7 +292,7 @@ impl EventVersion
                     Ok(BlobSize::U32) => Ok(api.read_u32().await? as usize),
                     Ok(BlobSize::U64) => Ok(api.read_u64().await? as usize),
                     Err(err) => {
-                        Err(SerializationError::IO(tokio::io::Error::new(tokio::io::ErrorKind::Other, format!("Failed to read data at 0x{:x} - {}", api.offset(), err))))
+                        Err(SerializationErrorKind::IO(tokio::io::Error::new(tokio::io::ErrorKind::Other, format!("Failed to read data at 0x{:x} - {}", api.offset(), err))).into())
                     }
                 }
             }
@@ -235,7 +324,7 @@ impl EventVersion
         match SerializationFormat::try_from(api.read_u8().await?) {
             Ok(a) => Ok(a),
             Err(_) => {
-                return Err(SerializationError::InvalidSerializationFormat);
+                Err(SerializationErrorKind::InvalidSerializationFormat.into())
             }
         }
     }
@@ -245,7 +334,7 @@ impl EventVersion
             EventVersion::V2 => {
                 match api.write_u8(format.into()).await {
                     Ok(_) => Ok(()),
-                    Err(err) => Err(SerializationError::IO(tokio::io::Error::new(tokio::io::ErrorKind::Other, format!("Failed to write data at 0x{:x} - {}", api.offset(), err))))
+                    Err(err) => Err(SerializationErrorKind::IO(tokio::io::Error::new(tokio::io::ErrorKind::Other, format!("Failed to write data at 0x{:x} - {}", api.offset(), err))).into())
                 }
             },
         }
