@@ -1,9 +1,10 @@
 #![allow(unused_imports)]
-use log::{error, info, debug};
+use tracing::{error, info, debug};
+use error_chain::bail;
 use std::fmt::Debug;
 use std::fmt::Formatter;
 use tokio::io;
-use tokio::time::Timeout;
+use crate::engine::timeout;
 use std::mem;
 use tokio::net;
 use tokio::net::UdpSocket;
@@ -12,7 +13,7 @@ use std::str;
 use std::time;
 use std::time::Duration;
 
-use crate::error::TimeError;
+use crate::error::*;
 
 const MODE_MASK: u8 = 0b0000_0111;
 const MODE_SHIFT: u8 = 0;
@@ -47,6 +48,8 @@ pub(crate) struct NtpResult {
     pub roundtrip: u64,
     /// Offset of the current system time with one received from a NTP server
     pub offset: i64,
+    /// Flag that indicates if this result is likely to be accurate or not
+    pub accurate: bool
 }
 
 impl NtpResult {
@@ -56,7 +59,7 @@ impl NtpResult {
     /// * `nsec` - number of nanoseconds
     /// * `roundtrip` - calculated roundtrip in microseconds
     /// * `offset` - calculated system clock offset in microseconds
-    pub(crate) fn new(sec: u32, nsec: u32, roundtrip: u64, offset: i64) -> Self {
+    pub(crate) fn new(sec: u32, nsec: u32, roundtrip: u64, offset: i64, accurate: bool) -> Self {
         let residue = nsec / NSEC_IN_SEC;
         let nsec = nsec % NSEC_IN_SEC;
         let sec = sec + residue;
@@ -66,6 +69,7 @@ impl NtpResult {
             nsec,
             roundtrip,
             offset,
+            accurate,
         }
     }
     /// Returns number of seconds reported by an NTP server
@@ -89,6 +93,12 @@ impl NtpResult {
     pub(crate) fn offset(&self) -> i64 {
         self.offset
     }
+
+    /// Returns if the result is accurate or not
+    #[allow(dead_code)]
+    pub(crate) fn accurate(&self) -> bool {
+        self.accurate
+    }
 }
 
 impl Debug for NtpResult {
@@ -234,7 +244,7 @@ pub(crate) async fn request(pool: &str, port: u16, timeout: time::Duration) -> i
     let req = NtpPacket::new();
     let dest = process_request(dest, &req, &socket).await?;
     let mut buf: RawNtpPacket = RawNtpPacket::default();
-    let (response, src) = tokio::time::timeout(timeout, socket.recv_from(buf.0.as_mut())).await??;
+    let (response, src) = crate::engine::timeout(timeout, socket.recv_from(buf.0.as_mut())).await??;
     let recv_timestamp = get_ntp_timestamp();
     
     if src != dest {
@@ -351,7 +361,7 @@ fn process_response(
     let nsec = (packet.tx_timestamp & MSEC_MASK) as u32;
     let tx_tm = seconds - NtpPacket::NTP_TIMESTAMP_DELTA;
 
-    Ok(NtpResult::new(tx_tm, nsec, delta.abs() as u64, theta))
+    Ok(NtpResult::new(tx_tm, nsec, delta.abs() as u64, theta, true))
 }
 
 fn convert_from_network(packet: &mut NtpPacket) {
@@ -386,11 +396,12 @@ pub(crate) async fn query_ntp(pool: &String, port: u16, tolerance_ms: u32) -> Re
     let ret = request(pool.as_str(), port, timeout).await?;
     let ping = Duration::from_micros(ret.roundtrip()).as_millis() as u32;
     if ping > tolerance_ms {
-        return Err(TimeError::BeyondTolerance(ping as u32));
+        bail!(TimeErrorKind::BeyondTolerance(ping as u32));
     }
     Ok(ret)
 }
 
+#[allow(dead_code)]
 pub(crate) async fn query_ntp_with_backoff(pool: &String, port: u16, tolerance_ms: u32, samples: u32) -> NtpResult
 {
     let mut wait_time = 50;
@@ -399,7 +410,7 @@ pub(crate) async fn query_ntp_with_backoff(pool: &String, port: u16, tolerance_m
             return result;
         }
 
-        tokio::time::sleep(Duration::from_millis(wait_time)).await;
+        crate::engine::sleep(Duration::from_millis(wait_time)).await;
         wait_time = (wait_time * 120) / 100;
         wait_time = wait_time + 50;
         if wait_time > 10000 {
@@ -421,11 +432,11 @@ pub(crate) async fn query_ntp_retry(pool: &String, port: u16, tolerance_ms: u32,
             None => Duration::from_millis(tolerance_ms as u64),
         };
 
-        #[cfg(feature = "super_verbose")]
-        debug!("ntp request timeout={}ms", timeout.as_millis());
+        #[cfg(feature = "enable_super_verbose")]
+        trace!("ntp request timeout={}ms", timeout.as_millis());
         if let Ok(ret) = request(pool.as_str(), port, timeout).await {
-            #[cfg(feature = "super_verbose")]
-            debug!("ntp response roundtrip={}, offset={}", ret.roundtrip, ret.offset);
+            #[cfg(feature = "enable_super_verbose")]
+            trace!("ntp response roundtrip={}, offset={}", ret.roundtrip, ret.offset);
 
             let current_ping = match &best {
                 Some(b) => b.roundtrip(),
@@ -442,7 +453,7 @@ pub(crate) async fn query_ntp_retry(pool: &String, port: u16, tolerance_ms: u32,
         }
         else
         {
-            tokio::time::sleep(Duration::from_millis(wait_time)).await;
+            crate::engine::sleep(Duration::from_millis(wait_time)).await;
             wait_time = (wait_time * 120) / 100;
             wait_time = wait_time + 50;
             if wait_time > 10000 {
