@@ -1,11 +1,12 @@
 #![allow(unused_imports)]
 use tokio::sync::mpsc as mpsc;
-use std::sync::mpsc as smpsc;
 use crate::meta::MetaParent;
 use fxhash::FxHashMap;
 use fxhash::FxHashSet;
 use std::sync::Arc;
-use parking_lot::RwLock as StdRwLock;
+use std::sync::RwLock as StdRwLock;
+use rcu_cell::RcuCell;
+use std::time::Duration;
 
 use super::crypto::AteHash;
 use super::event::*;
@@ -33,19 +34,33 @@ pub enum TransactionScope
     Full
 }
 
+impl std::fmt::Display
+for TransactionScope
+{
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        match self {
+            TransactionScope::None => write!(f, "none"),
+            TransactionScope::Local => write!(f, "local"),
+            TransactionScope::Full => write!(f, "full"),
+        }
+    }
+}
+
 #[derive(Debug, Default)]
 pub struct ConversationSession
 {
-    pub force_centralized_mode: bool,
-    pub other_end_is_server: bool,
+    pub id: RcuCell<AteHash>,
+    pub weaken_validation: bool,
     pub signatures: StdRwLock<FxHashSet<AteHash>>,
 }
 
 impl ConversationSession {
-    pub fn new(other_end_is_server: bool) -> ConversationSession {
-        let mut ret = ConversationSession::default();
-        ret.other_end_is_server = other_end_is_server;
-        ret
+    pub fn clear(&self) {
+        if let Some(mut guard) = self.id.try_lock() {
+            guard.update(None);
+        }
+        let mut guard = self.signatures.write().unwrap();
+        guard.clear();
     }
 }
 
@@ -55,18 +70,20 @@ pub(crate) struct Transaction
     pub(crate) scope: TransactionScope,
     pub(crate) transmit: bool,
     pub(crate) events: Vec<EventData>,
+    pub(crate) timeout: Duration,
     pub(crate) conversation: Option<Arc<ConversationSession>>,
 }
 
 impl Transaction
 {
     #[allow(dead_code)]
-    pub(crate) fn from_events(events: Vec<EventData>, scope: TransactionScope, transmit: bool) -> Transaction
+    pub(crate) fn from_events(events: Vec<EventData>, scope: TransactionScope, transmit: bool, timeout: Duration) -> Transaction
     {
         Transaction {
             scope,
             transmit,
             events,
+            timeout,
             conversation: None,
         }
     }
