@@ -6,21 +6,16 @@ use super::transaction::TransactionMetadata;
 use snap::read::FrameDecoder;
 use snap::read::FrameEncoder;
 use bytes::{Bytes, Buf};
-#[allow(unused_imports)]
-use openssl::symm::{encrypt, decrypt, Cipher};
-
-#[cfg(test)]
-use super::conf::ConfAte;
 
 pub trait EventDataTransformer: Send + Sync
 {
     /// Callback when data is stored in the event 
-    fn data_as_underlay(&self, _meta: &mut Metadata, with: Bytes, _session: &AteSession, _trans_meta: &TransactionMetadata) -> Result<Bytes, TransformError> {
+    fn data_as_underlay(&self, _meta: &mut Metadata, with: Bytes, _session: &'_ dyn AteSession, _trans_meta: &TransactionMetadata) -> Result<Bytes, TransformError> {
         Ok(with)
     }
 
     /// Callback before data in an event is actually used by an actual user
-    fn data_as_overlay(&self, _meta: &Metadata, with: Bytes, _session: &AteSession) -> Result<Bytes, TransformError> {
+    fn data_as_overlay(&self, _meta: &Metadata, with: Bytes, _session: &'_ dyn AteSession) -> Result<Bytes, TransformError> {
         Ok(with)
     }
 
@@ -40,7 +35,7 @@ for CompressorWithSnapTransformer
     }
 
     #[allow(unused_variables)]
-    fn data_as_underlay(&self, meta: &mut Metadata, with: Bytes, _session: &AteSession, _trans_meta: &TransactionMetadata) -> Result<Bytes, TransformError> {
+    fn data_as_underlay(&self, meta: &mut Metadata, with: Bytes, _session: &'_ dyn AteSession, _trans_meta: &TransactionMetadata) -> Result<Bytes, TransformError> {
         let mut reader = FrameEncoder::new(with.reader());
         let mut compressed = Vec::new();
         std::io::copy(&mut reader, &mut compressed)?;
@@ -48,7 +43,7 @@ for CompressorWithSnapTransformer
     }
 
     #[allow(unused_variables)]
-    fn data_as_overlay(&self, meta: &Metadata, with: Bytes, _session: &AteSession) -> Result<Bytes, TransformError> {
+    fn data_as_overlay(&self, meta: &Metadata, with: Bytes, _session: &'_ dyn AteSession) -> Result<Bytes, TransformError> {
         let mut reader = FrameDecoder::new(with.reader());
         let mut decompressed = Vec::new();
         std::io::copy(&mut reader, &mut decompressed)?;
@@ -80,18 +75,18 @@ for StaticEncryptionTransformer
     }
     
     #[allow(unused_variables)]
-    fn data_as_underlay(&self, meta: &mut Metadata, with: Bytes, _session: &AteSession, _trans_meta: &TransactionMetadata) -> Result<Bytes, TransformError>
+    fn data_as_underlay(&self, meta: &mut Metadata, with: Bytes, _session: &'_ dyn AteSession, _trans_meta: &TransactionMetadata) -> Result<Bytes, TransformError>
     {
         let iv = meta.generate_iv();
-        let encrypted = self.key.encrypt_with_iv(&iv, &with[..])?;
+        let encrypted = self.key.encrypt_with_iv(&iv, &with[..]);
         Ok(Bytes::from(encrypted))
     }
 
     #[allow(unused_variables)]
-    fn data_as_overlay(&self, meta: &Metadata, with: Bytes, _session: &AteSession) -> Result<Bytes, TransformError>
+    fn data_as_overlay(&self, meta: &Metadata, with: Bytes, _session: &'_ dyn AteSession) -> Result<Bytes, TransformError>
     {
         let iv = meta.get_iv()?;
-        let decrypted = self.key.decrypt(&iv, &with[..])?;
+        let decrypted = self.key.decrypt(&iv, &with[..]);
         Ok(Bytes::from(decrypted))
     }
 }
@@ -99,23 +94,22 @@ for StaticEncryptionTransformer
 #[test]
 fn test_encrypter()
 {
-    crate::utils::bootstrap_env();
+    crate::utils::bootstrap_test_env();
 
-    let key = EncryptKey::from_seed_string("test".to_string(), KeySize::Bit256);
+    let key = EncryptKey::from_seed_string("test".to_string(), KeySize::Bit192);
     let encrypter = StaticEncryptionTransformer::new(&key);
-    let cfg = ConfAte::default();
-
+    
     let trans_meta = TransactionMetadata::default();
     let test_bytes = Bytes::from_static(b"Some Crypto Text");
     let mut meta = Metadata::default();
-    let encrypted = encrypter.data_as_underlay(&mut meta, test_bytes.clone(), &AteSession::new(&cfg), &trans_meta).unwrap();
+    let encrypted = encrypter.data_as_underlay(&mut meta, test_bytes.clone(), &AteSessionUser::new(), &trans_meta).unwrap();
 
     println!("metadata: {:?}", meta);
     println!("data_test: {:X}", &test_bytes);
     println!("data_encrypted: {:X}", &encrypted);
     assert_ne!(&test_bytes, &encrypted);
     
-    let decrypted = encrypter.data_as_overlay(&mut meta, encrypted, &AteSession::new(&cfg)).unwrap();
+    let decrypted = encrypter.data_as_overlay(&mut meta, encrypted, &AteSessionUser::new()).unwrap();
 
     println!("data_decrypted: {:X}", &decrypted);
     assert_eq!(&test_bytes, &decrypted);
@@ -124,22 +118,21 @@ fn test_encrypter()
 #[test]
 fn test_compressor()
 {
-    crate::utils::bootstrap_env();
+    crate::utils::bootstrap_test_env();
 
     let compressor = CompressorWithSnapTransformer::default();
-    let cfg = ConfAte::default();
-
+    
     let trans_meta = TransactionMetadata::default();
     let test_bytes = Bytes::from("test".as_bytes());
     let mut meta = Metadata::default();
-    let compressed = compressor.data_as_underlay(&mut meta, test_bytes.clone(), &AteSession::new(&cfg), &trans_meta).unwrap();
+    let compressed = compressor.data_as_underlay(&mut meta, test_bytes.clone(), &AteSessionUser::new(), &trans_meta).unwrap();
 
     println!("metadata: {:?}", meta);
     println!("data_test: {:X}", &test_bytes);
     println!("data_compressed: {:X}", &compressed);
     assert_ne!(&test_bytes, &compressed);
     
-    let decompressed = compressor.data_as_overlay(&mut meta, compressed, &AteSession::new(&cfg)).unwrap();
+    let decompressed = compressor.data_as_overlay(&mut meta, compressed, &AteSessionUser::new()).unwrap();
 
     println!("data_decompressed: {:X}", &decompressed);
     assert_eq!(&test_bytes, &decompressed);
@@ -148,35 +141,24 @@ fn test_compressor()
 #[test]
 fn test_crypto()
 {
-    crate::utils::bootstrap_env();
+    crate::utils::bootstrap_test_env();
     
-    let cipher = Cipher::aes_128_cbc();
-    let data = b"Some Crypto Text";
     let key = b"\x00\x01\x02\x03\x04\x05\x06\x07\x08\x09\x0A\x0B\x0C\x0D\x0E\x0F";
+    let cipher = crate::crypto::EncryptKey::Aes128(key.clone());
+    let data = b"Some Crypto Text";
     let iv = b"\x00\x01\x02\x03\x04\x05\x06\x07\x00\x01\x02\x03\x04\x05\x06\x07";
-    let ciphertext = encrypt(
-        cipher,
-        key,
-        Some(iv),
-        data).unwrap();
+    let ciphertext = cipher.encrypt_with_iv(&InitializationVector::from(iv), data);
 
     assert_eq!(
-        b"\xB4\xB9\xE7\x30\xD6\xD6\xF7\xDE\x77\x3F\x1C\xFF\xB3\x3E\x44\x5A\x91\xD7\x27\x62\x87\x4D\
-        \xFB\x3C\x5E\xC4\x59\x72\x4A\xF4\x7C\xA1",
+        [110, 148, 177, 161, 48, 153, 25, 114, 206, 212, 126, 250, 70, 201, 154, 141],
         &ciphertext[..]);
 
-    let cipher = Cipher::aes_128_cbc();
-    let data = b"\xB4\xB9\xE7\x30\xD6\xD6\xF7\xDE\x77\x3F\x1C\xFF\xB3\x3E\x44\x5A\x91\xD7\x27\x62\
-                \x87\x4D\xFB\x3C\x5E\xC4\x59\x72\x4A\xF4\x7C\xA1";
     let key = b"\x00\x01\x02\x03\x04\x05\x06\x07\x08\x09\x0A\x0B\x0C\x0D\x0E\x0F";
+    let cipher = crate::crypto::EncryptKey::Aes128(key.clone());
     let iv = b"\x00\x01\x02\x03\x04\x05\x06\x07\x00\x01\x02\x03\x04\x05\x06\x07";
-    let ciphertext = decrypt(
-        cipher,
-        key,
-        Some(iv),
-        data).unwrap();
-
+    let data = cipher.decrypt(&InitializationVector::from(iv), &ciphertext[..]);
+    
     assert_eq!(
         b"Some Crypto Text",
-        &ciphertext[..]);
+        &data[..]);
 }
\ No newline at end of file
