@@ -1,5 +1,5 @@
 #[allow(unused_imports)]
-use log::{error, info, warn, debug};
+use tracing::{error, info, warn, debug};
 
 use crate::crypto::*;
 use crate::error::*;
@@ -10,17 +10,17 @@ use super::*;
 
 impl TreeAuthorityPlugin
 {
-    pub(super) fn generate_encrypt_key(&self, auth: &ReadOption, session: &AteSession) -> Result<Option<(InitializationVector, EncryptKey)>, TransformError>
+    pub(super) fn generate_encrypt_key(&self, auth: &ReadOption, session: &'_ dyn AteSession) -> Result<Option<(InitializationVector, EncryptKey)>, TransformError>
     {
         match auth {
             ReadOption::Inherit => {
-                Err(TransformError::UnspecifiedReadability)
+                Err(TransformErrorKind::UnspecifiedReadability.into())
             },
             ReadOption::Everyone(_key) => {
                 Ok(None)
             },
             ReadOption::Specific(key_hash, derived) => {
-                for key in session.read_keys() {
+                for key in session.read_keys(AteSessionKeyCategory::AllKeys) {
                     if key.hash() == *key_hash {
                         return Ok(Some((
                             InitializationVector::generate(),
@@ -28,7 +28,7 @@ impl TreeAuthorityPlugin
                         )));
                     }
                 }
-                for key in session.private_read_keys() {
+                for key in session.private_read_keys(AteSessionKeyCategory::AllKeys) {
                     if key.hash() == *key_hash {
                         return Ok(Some((
                             InitializationVector::generate(),
@@ -36,7 +36,7 @@ impl TreeAuthorityPlugin
                         )));
                     }
                 }
-                Err(TransformError::MissingReadKey(key_hash.clone()))
+                Err(TransformErrorKind::MissingReadKey(key_hash.clone()).into())
             }
         }
     }
