@@ -1,6 +1,9 @@
 #[allow(unused_imports)]
-use log::{info, error, debug};
+use tracing::{info, warn, debug, error, trace, instrument, span, Level};
+use std::sync::Mutex as StdMutex;
+use std::sync::Arc;
 
+use crate::comms::Metrics;
 use crate::meta::*;
 use crate::error::*;
 use crate::header::*;
@@ -12,16 +15,26 @@ use super::*;
 
 pub(crate) struct ChainOfTrust
 {
+    pub(crate) debug_id: u64,
     pub(crate) key: ChainKey,
     pub(crate) timeline: ChainTimeline,
     pub(crate) redo: RedoLog,
+    pub metrics: Arc<StdMutex<Metrics>>,
+}
+
+impl std::fmt::Debug
+for ChainOfTrust
+{
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        write!(f, "chain-of-trust(key={},debug_id={})", self.key, self.debug_id)
+    }
 }
 
 impl<'a> ChainOfTrust
 {
     pub(crate) async fn load(&self, leaf: EventLeaf) -> Result<LoadResult, LoadError>
     {
-        #[cfg(feature = "verbose")]
+        #[cfg(feature = "enable_verbose")]
         debug!("loading: {}", leaf.record);
         
         let data = self.redo.load(leaf.record.clone()).await?;
@@ -92,7 +105,13 @@ impl<'a> ChainOfTrust
         self.key.name.clone()
     }
 
-    pub(crate) fn add_history(&mut self, header: &EventHeader) {
+    pub(crate) fn add_history(&mut self, header: EventHeader)
+    {
+        {
+            let mut metrics = self.metrics.lock().unwrap();
+            metrics.chain_size += header.raw.meta_bytes.len() as u64;
+            metrics.chain_size += header.raw.data_size as u64;
+        }
         self.timeline.add_history(header)
     }
 }
\ No newline at end of file
