@@ -1,11 +1,12 @@
 pub mod chain_key;
+pub mod chain_ref;
 pub mod chain_of_trust;
-pub mod integrity_mode;
 pub mod load_result;
 pub mod tests;
 pub mod header;
 pub mod timeline;
 
+#[allow(unused_imports)]
 #[cfg(test)]
 pub(crate) use tests::*;
 
@@ -13,6 +14,6 @@ pub(crate) use chain_of_trust::*;
 pub(crate) use timeline::*;
 
 pub use chain_key::*;
-pub use integrity_mode::*;
+pub use chain_ref::*;
 pub use load_result::*;
 pub use header::*;
\ No newline at end of file
