@@ -1,19 +1,24 @@
+#![cfg(any(feature = "enable_full"))]
 #![allow(unused_imports)]
 use ate::prelude::*;
-use tokio::runtime::Runtime;
 
-#[cfg(feature = "rotate")]
+#[cfg(feature = "enable_server")]
+#[cfg(feature = "enable_rotate")]
 #[test]
 fn rotate_test() -> Result<(), AteError>
 {
-    ate::utils::bootstrap_env();
+    ate::utils::bootstrap_test_env();
 
-    let rt = Runtime::new().unwrap();
-    rt.block_on(async {
+    #[cfg(feature = "enable_mt")]
+    let rt = tokio::runtime::Builder::new_multi_thread().enable_all().build()?;
+    #[cfg(not(feature = "enable_mt"))]
+    let rt =tokio::runtime::Builder::new_current_thread().enable_all().build()?;
+
+    rt.block_on(TaskEngine::run_until(async {
 
         // The default configuration will store the redo log locally in the temporary folder
         let mut conf = ConfAte::default();
-        #[cfg(feature = "local_fs")]
+        #[cfg(feature = "enable_local_fs")]
         {
             conf.log_path = Some("/tmp/ate".to_string());
         }
@@ -25,11 +30,11 @@ fn rotate_test() -> Result<(), AteError>
         {
             // We create a chain with a specific key (this is used for the file name it creates)
             let chain = builder.open(&ChainKey::from("rotate")).await?;
-            let session = AteSession::new(&conf);
+            let session = AteSessionUser::new();
 
             {
                 // Write a test object
-                let mut dio = chain.dio(&session).await;
+                let dio = chain.dio_mut(&session).await;
                 key1 = dio.store("blah!".to_string())?.key().clone();
                 dio.commit().await?;
             }
@@ -39,12 +44,12 @@ fn rotate_test() -> Result<(), AteError>
 
             {
                 // Write a test object
-                let mut dio = chain.dio(&session).await;
+                let dio = chain.dio_mut(&session).await;
                 key2 = dio.store("haha!".to_string())?.key().clone();
                 dio.commit().await?;
             }
 
-            let mut dio = chain.dio(&session).await;
+            let dio = chain.dio(&session).await;
             assert_eq!(*dio.load::<String>(&key1).await?, "blah!".to_string());
             assert_eq!(*dio.load::<String>(&key2).await?, "haha!".to_string());
         }
@@ -52,8 +57,8 @@ fn rotate_test() -> Result<(), AteError>
         {
             let chain = builder.open(&ChainKey::from("rotate")).await?;
 
-            let session = AteSession::new(&conf);
-            let mut dio = chain.dio(&session).await;
+            let session = AteSessionUser::new();
+            let dio = chain.dio(&session).await;
             assert_eq!(*dio.load::<String>(&key1).await?, "blah!".to_string());
             assert_eq!(*dio.load::<String>(&key2).await?, "haha!".to_string());
             
@@ -61,5 +66,5 @@ fn rotate_test() -> Result<(), AteError>
         }
 
         Ok(())
-    })
+    }))
 }
\ No newline at end of file
