@@ -1,5 +1,6 @@
+#![cfg(any(feature = "enable_full"))]
 #![allow(unused_imports)]
-use log::{error, info, warn, debug};
+use tracing::{error, info, warn, debug};
 use serde::{Serialize, Deserialize};
 use ate::prelude::*;
 use rust_decimal::prelude::*;
@@ -18,17 +19,21 @@ struct Garage
     cars: DaoVec<Car>,
 }
 
-#[cfg(feature = "local_fs")]
+#[cfg(any(feature = "enable_server", feature = "enable_client" ))]
+#[cfg(feature = "enable_local_fs")]
 #[test]
 fn test_trust_tree_persistent() -> Result<(), AteError>
 {
-    ate::utils::bootstrap_env();
+    ate::utils::bootstrap_test_env();
+
+    #[cfg(feature = "enable_mt")]
+    let rt = tokio::runtime::Builder::new_multi_thread().enable_all().build()?;
+    #[cfg(not(feature = "enable_mt"))]
+    let rt =tokio::runtime::Builder::new_current_thread().enable_all().build()?;
 
-    let rt = Runtime::new().unwrap();
     rt.block_on(async
     {
-
-        debug!("generating crypto keys");
+        info!("generating crypto keys");
         let write_key = PrivateSignKey::generate(KeySize::Bit192);
         let write_key2 = PrivateSignKey::generate(KeySize::Bit256);
         let read_key = EncryptKey::generate(KeySize::Bit256);
@@ -41,14 +46,14 @@ fn test_trust_tree_persistent() -> Result<(), AteError>
 
         let key1;
         {
-            debug!("building the session");
-            let mut session = AteSession::new(&conf);    
+            info!("building the session");
+            let mut session = AteSessionUser::new();    
             session.user.properties.push(AteSessionProperty::WriteKey(write_key.clone()));
             session.user.properties.push(AteSessionProperty::WriteKey(write_key2.clone()));
             session.user.properties.push(AteSessionProperty::ReadKey(read_key.clone()));
-            session.user.properties.push(AteSessionProperty::Identity("author@here.com".to_string()));
+            session.identity = "author@here.com".to_string();
 
-            debug!("creating the chain-of-trust");
+            info!("creating the chain-of-trust");
             let builder = ChainBuilder::new(&conf)
                 .await
                 .add_root_public_key(&root_public_key)
@@ -56,10 +61,10 @@ fn test_trust_tree_persistent() -> Result<(), AteError>
                 .build();
             let chain = builder.open(&ChainKey::from("trust")).await?;
 
-            debug!("add the objects to the DIO");
-            let mut dio = chain.dio(&session).await;
+            info!("add the objects to the DIO");
+            let dio = chain.dio_mut(&session).await;
             let mut garage = dio.store(Garage::default())?;
-            garage.auth_mut().read = ReadOption::from_key(&read_key)?;
+            garage.auth_mut().read = ReadOption::from_key(&read_key);
             garage.auth_mut().write = WriteOption::Specific(write_key2.hash());
             
             for n in 0..100 {
@@ -68,10 +73,9 @@ fn test_trust_tree_persistent() -> Result<(), AteError>
                 let mut car = Car::default();
                 car.name = name.clone();
                 
-                let car = garage.push_store(&mut dio, garage.cars, car)?;
+                let car = garage.as_mut().cars.push(car)?;
                 assert_eq!(car.name, name);
             }
-            garage.commit(&mut dio)?;
             dio.commit().await?;
             drop(dio);
 
@@ -79,14 +83,14 @@ fn test_trust_tree_persistent() -> Result<(), AteError>
         }
 
         {
-            debug!("building the session");
-            let mut session = AteSession::new(&conf);    
+            info!("building the session");
+            let mut session = AteSessionUser::new();    
             session.user.properties.push(AteSessionProperty::WriteKey(write_key2.clone()));
             session.user.properties.push(AteSessionProperty::ReadKey(read_key.clone()));
-            session.user.properties.push(AteSessionProperty::Identity("author@here.com".to_string()));
+            session.identity = "author@here.com".to_string();
 
             let chain = {
-                debug!("loading the chain-of-trust again");
+                info!("loading the chain-of-trust again");
                 let mut conf = ConfAte::default();
                 conf.log_path = Some("/tmp/ate".to_string());
                 conf.log_format.meta = SerializationFormat::Json;
@@ -99,9 +103,9 @@ fn test_trust_tree_persistent() -> Result<(), AteError>
             };
 
             // Load the garage
-            let mut dio = chain.dio(&session).await;
+            let dio = chain.dio(&session).await;
             let garage = dio.load::<Garage>(&key1).await?;
-            assert_eq!(garage.iter(&mut dio, garage.cars).await?.collect::<Vec<_>>().len(), 100);
+            assert_eq!(garage.cars.iter().await?.count(), 100);
 
             // Delete the chain
             chain.single().await.destroy().await.unwrap();
@@ -111,16 +115,20 @@ fn test_trust_tree_persistent() -> Result<(), AteError>
     })
 }
 
+#[cfg(any(feature = "enable_server", feature = "enable_client" ))]
 #[test]
 fn test_trust_tree_memory() -> Result<(), AteError>
 {
-    ate::utils::bootstrap_env();
+    ate::utils::bootstrap_test_env();
 
-    let rt = Runtime::new().unwrap();
-    rt.block_on(async
+    #[cfg(feature = "enable_mt")]
+    let rt = tokio::runtime::Builder::new_multi_thread().enable_all().build()?;
+    #[cfg(not(feature = "enable_mt"))]
+    let rt =tokio::runtime::Builder::new_current_thread().enable_all().build()?;
+
+    rt.block_on(TaskEngine::run_until(async
     {
-
-        debug!("generating crypto keys");
+        info!("generating crypto keys");
         let write_key = PrivateSignKey::generate(KeySize::Bit192);
         let write_key2 = PrivateSignKey::generate(KeySize::Bit256);
         let read_key = EncryptKey::generate(KeySize::Bit256);
@@ -132,14 +140,14 @@ fn test_trust_tree_memory() -> Result<(), AteError>
 
         let key1;
         {
-            debug!("building the session");
-            let mut session = AteSession::new(&conf);    
+            info!("building the session");
+            let mut session = AteSessionUser::new();    
             session.user.properties.push(AteSessionProperty::WriteKey(write_key.clone()));
             session.user.properties.push(AteSessionProperty::WriteKey(write_key2.clone()));
             session.user.properties.push(AteSessionProperty::ReadKey(read_key.clone()));
-            session.user.properties.push(AteSessionProperty::Identity("author@here.com".to_string()));
+            session.identity = "author@here.com".to_string();
 
-            debug!("creating the chain-of-trust");
+            info!("creating the chain-of-trust");
             let builder = ChainBuilder::new(&conf)
                 .await
                 .add_root_public_key(&root_public_key)
@@ -147,10 +155,10 @@ fn test_trust_tree_memory() -> Result<(), AteError>
                 .build();
             let chain = builder.open(&ChainKey::from("trust")).await?;
 
-            debug!("add the objects to the DIO");
-            let mut dio = chain.dio(&session).await;
+            info!("add the objects to the DIO");
+            let dio = chain.dio_mut(&session).await;
             let mut garage = dio.store(Garage::default())?;
-            garage.auth_mut().read = ReadOption::from_key(&read_key)?;
+            garage.auth_mut().read = ReadOption::from_key(&read_key);
             garage.auth_mut().write = WriteOption::Specific(write_key2.hash());
             
             for n in 0..100 {
@@ -159,30 +167,29 @@ fn test_trust_tree_memory() -> Result<(), AteError>
                 let mut car = Car::default();
                 car.name = name.clone();
                 
-                let car = garage.push_store(&mut dio, garage.cars, car)?;
+                let car = garage.as_mut().cars.push(car)?;
                 assert_eq!(car.name, name);
             }
-            garage.commit(&mut dio)?;
             dio.commit().await?;
             drop(dio);
 
             key1 = garage.key().clone();
 
-            debug!("building the session");
-            let mut session = AteSession::new(&conf);    
+            info!("building the session");
+            let mut session = AteSessionUser::new();    
             session.user.properties.push(AteSessionProperty::WriteKey(write_key2.clone()));
             session.user.properties.push(AteSessionProperty::ReadKey(read_key.clone()));
-            session.user.properties.push(AteSessionProperty::Identity("author@here.com".to_string()));
+            session.identity = "author@here.com".to_string();
 
             // Load the garage
-            let mut dio = chain.dio(&session).await;
+            let dio = chain.dio(&session).await;
             let garage = dio.load::<Garage>(&key1).await?;
-            assert_eq!(garage.iter(&mut dio, garage.cars).await?.collect::<Vec<_>>().len(), 100);
+            assert_eq!(garage.cars.iter().await?.count(), 100);
 
             // Delete the chain
             chain.single().await.destroy().await.unwrap();
         }
 
         Ok(())
-    })
+    }))
 }
\ No newline at end of file
